/*
  gpslogger.cpp
  Copyright (C) 2008 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

#include "qgmap.h"
#include "gpslogger.h"
#include "pos.h"

#include <qstring.h>
#include <qfile.h>
#include <qdatetime.h>

#include <pthread.h>
#include <signal.h>
#include <unistd.h>
#include <stdlib.h>

static void *GPSLogger_thread(void *arg);

GPSLogger::GPSLogger()
{
	thread_run = false;
	started = false;
	dev = NULL;
}

GPSLogger::~GPSLogger()
{
}

GPSInfo GPSLogger::getInfo()
{
	return gpsinfo.copy();
}

QString GPSLogger::getErrmsg()
{
	return errmsg;
}

bool GPSLogger::isLive()
{
	return thread_run;
}

bool GPSLogger::isStarted()
{
	return started;
}

int GPSLogger::setDeviceFile(QString const &dev)
{
	if(thread_run)
		return 1;

	dev_filename = dev;
	return 0;
}

void GPSLogger::setInitCommand(QString const &cmd)
{
	init_cmd = cmd;
}

void GPSLogger::setEndCommand(QString const &cmd)
{
	end_cmd = cmd;
}

int GPSLogger::start()
{
	int ret;
	dev = new QFile(dev_filename);

	if(thread_run){
		errmsg = QString::fromUtf8("既に起動しています");
		return 1;
	}

	if (!dev->exists()) {
		errmsg = QString::fromUtf8("デバイスファイルが存在しません");
		return 1;
	}

	if(init_cmd != "")
		ret = system(init_cmd.ascii());

	thread_run = true;
	started = false;
	pthread_create(&tpid, NULL, GPSLogger_thread, (void *)this);
	return 0;
}

int GPSLogger::stop()
{
	int ret;

	if(!thread_run)
		return 1;

	thread_run = false;
	sleep(1);
	pthread_cancel(tpid);
	pthread_join(tpid, NULL);
	started = false;

	dev->close();
	delete dev;

	dev = NULL;

	if(end_cmd != "")
		ret = system(end_cmd.ascii());
	return 0;
}

void GPSLogger::thread()
{
	QString line;

	if (!dev->open(IO_ReadOnly)) {
		errmsg = QString::fromUtf8("デバイスファイルが開けません");

		thread_run = false;
	}

	started = true;

	while(thread_run){
		if(dev->atEnd()){
			usleep(100000);  // sleep 0.1s
			continue;
		}

		dev->readLine(line, 512);

		// XXX Todo: logging to file

		// update GPS Information
		gpsinfo.updateInfo(line);
	}
}

static void *GPSLogger_thread(void *arg)
{
	GPSLogger *gps = (GPSLogger *)arg;
	gps->thread();
	return NULL;
}
