/*
  pos.h
  Copyright (C) 2008 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __POS_H__
#define __POS_H__

#define MAX_ZOOM 19
#define MIN_ZOOM 0

#include <qpoint.h>

class QString;

class Pos {
 public:
  Pos(double _lat = 35.471226, double _lon = 139.619236, int _zoom = 13);
  Pos(int _x, int _y, int _zoom);

  void setPos(int _x, int _y);
  void setZoom(int _zoom);
  void setLatLon(double _lat, double _lon, bool wgs = true);
  QString getPos();
  bool setPos(QString const &str, bool wgs = true);

  Pos &offset(int _x, int _y);
  Pos &offset_clip(int _x, int _y);
  bool zoomup(int zoom);
  bool zoomTo(int zoom);

  void normalize();
  void clipY();

  int getX() const;
  int getY() const;
  int getZoom() const;

  double getLat();
  double getLon();

  double getLatLength();
  double getDotLength();
  int getScaleSize(int max, int *mul);

  QPoint getPoint();

  QString serialize();
  bool deserialize(QString const &str);

  bool operator==(const Pos &pos) const;
  bool operator!=(const Pos &pos) const;

  static void convTokyoToWGS(double *wgs_lat, double *wgs_lon,
							 double tokyo_lat, double tokyo_lon){
	  double lat, lon;
	  lat = tokyo_lat - 0.000106950 * tokyo_lat + 0.000017464 * tokyo_lon + 0.0046017;
	  lon = tokyo_lon - 0.000046038 * tokyo_lat - 0.000083043 * tokyo_lon + 0.010040;
	  *wgs_lat = lat;
	  *wgs_lon = lon;
  }

 private:
  int x;
  int y;
  int zoom;		// zoom lebel: 0(micro)-19(macro)

  bool parseStr(const QString str, double *value);
};

#endif // __POS_H__

